@@ -9,6 +9,7 @@ import nose
 from fabric.api import *
 
 import docs, tag
+from utils import msg
 
 
 @task(default=True)
@@ -20,10 +21,7 @@ def test(args=None):
     """
     default_args = "-sv --with-doctest --nologcapture --with-color"
     default_args += (" " + args) if args else ""
-    try:
-        nose.core.run(argv=[''] + default_args.split())
-    except SystemExit:
-        abort("Nose encountered an error; you may be missing newly added test dependencies. Try running 'pip install -r requirements.txt'.")
+    nose.core.run_exit(argv=[''] + default_args.split())
 
 
 @task
@@ -31,7 +29,8 @@ def upload():
     """
     Build, register and upload to PyPI
     """
-    local('python setup.py sdist register upload')
+    with msg("Uploading to PyPI"):
+        local('python setup.py sdist register upload')
 
 
 @task
