@@ -6,6 +6,7 @@ from __future__ import with_statement
 
 import os
 import os.path
+import posixpath
 import re
 import stat
 import subprocess
@@ -531,7 +532,7 @@ def get(remote_path, local_path=None):
             # Otherwise, be relative to remote home directory (SFTP server's
             # '.')
             else:
-                remote_path = os.path.join(home, remote_path)
+                remote_path = posixpath.join(home, remote_path)
 
         # Track final local destination files so we can return a list
         local_files = []
@@ -578,7 +579,7 @@ def _sudo_prefix(user):
     Return ``env.sudo_prefix`` with ``user`` inserted if necessary.
     """
     # Insert env.sudo_prompt into env.sudo_prefix
-    prefix = env.sudo_prefix % env.sudo_prompt
+    prefix = env.sudo_prefix % env
     if user is not None:
         if str(user).isdigit():
             user = "#%s" % user
@@ -697,8 +698,9 @@ def _execute(channel, command, pty=True, combine_stderr=None,
 
         # Use SSH agent forwarding from 'ssh' if enabled by user
         config_agent = ssh_config().get('forwardagent', 'no').lower() == 'yes'
+        forward = None
         if env.forward_agent or config_agent:
-            forward = ssh.agent.AgentClientProxy(channel)
+            forward = ssh.agent.AgentRequestHandler(channel)
 
         # Kick off remote command
         if invoke_shell:
@@ -739,6 +741,9 @@ def _execute(channel, command, pty=True, combine_stderr=None,
 
         # Close channel
         channel.close()
+        # Close any agent forward proxies
+        if forward is not None:
+            forward.close()
 
         # Update stdout/stderr with captured values if applicable
         if not invoke_shell:
@@ -948,15 +953,17 @@ def local(command, capture=False):
     When ``capture=False``, the local subprocess' stdout and stderr streams are
     hooked up directly to your terminal, though you may use the global
     :doc:`output controls </usage/output_controls>` ``output.stdout`` and
-    ``output.stderr`` to hide one or both if desired. In this mode,
-    `~fabric.operations.local` returns None.
-
-    When ``capture=True``, this function will return the contents of the
-    command's stdout as a string-like object; as with `~fabric.operations.run`
-    and `~fabric.operations.sudo`, this return value exhibits the
-    ``return_code``, ``stderr``, ``failed`` and ``succeeded`` attributes. See
-    `run` for details.
+    ``output.stderr`` to hide one or both if desired. In this mode, the return
+    value's stdout/stderr values are always empty.
 
+    When ``capture=True``, you will not see any output from the subprocess in
+    your terminal, but the return value will contain the captured
+    stdout/stderr.
+    
+    In either case, as with `~fabric.operations.run` and
+    `~fabric.operations.sudo`, this return value exhibits the ``return_code``,
+    ``stderr``, ``failed`` and ``succeeded`` attributes. See `run` for details.
+    
     `~fabric.operations.local` will honor the `~fabric.context_managers.lcd`
     context manager, allowing you to control its current working directory
     independently of the remote end (which honors
@@ -1004,7 +1011,7 @@ def local(command, capture=False):
     if p.returncode != 0:
         out.failed = True
         msg = "local() encountered an error (return code %s) while executing '%s'" % (p.returncode, command)
-        error(message=msg)
+        error(message=msg, stdout=out, stderr=err)
     out.succeeded = not out.failed
     # If we were capturing, this will be a string; otherwise it will be None.
     return out
