@@ -282,7 +282,7 @@ class TestNetwork(FabricTest):
         output.everything = False
         with password_response(PASSWORDS[env.user], silent=False):
             run("ls /simple")
-        regex = r'^\[%s\] Login password: ' % env.host_string
+        regex = r'^\[%s\] Login password for \'%s\': ' % (env.host_string, env.user)
         assert_contains(regex, sys.stderr.getvalue())
 
     @mock_streams('stderr')
@@ -297,7 +297,7 @@ class TestNetwork(FabricTest):
         output.everything = False
         with password_response(CLIENT_PRIVKEY_PASSPHRASE, silent=False):
             run("ls /simple")
-        regex = r'^\[%s\] Login password: ' % env.host_string
+        regex = r'^\[%s\] Login password for \'%s\': ' % (env.host_string, env.user)
         assert_contains(regex, sys.stderr.getvalue())
 
     def test_sudo_prompt_display_passthrough(self):
@@ -328,20 +328,23 @@ class TestNetwork(FabricTest):
         if display_output:
             expected = """
 [%(prefix)s] sudo: oneliner
-[%(prefix)s] Login password: 
+[%(prefix)s] Login password for '%(user)s': 
 [%(prefix)s] out: sudo password:
 [%(prefix)s] out: Sorry, try again.
 [%(prefix)s] out: sudo password: 
 [%(prefix)s] out: result
-""" % {'prefix': env.host_string}
+""" % {'prefix': env.host_string, 'user': env.user}
         else:
             # Note lack of first sudo prompt (as it's autoresponded to) and of
             # course the actual result output.
             expected = """
 [%(prefix)s] sudo: oneliner
-[%(prefix)s] Login password: 
+[%(prefix)s] Login password for '%(user)s': 
 [%(prefix)s] out: Sorry, try again.
-[%(prefix)s] out: sudo password: """ % {'prefix': env.host_string}
+[%(prefix)s] out: sudo password: """ % {
+    'prefix': env.host_string,
+    'user': env.user
+}
         eq_(expected[1:], sys.stdall.getvalue())
 
     @mock_streams('both')
@@ -364,7 +367,7 @@ class TestNetwork(FabricTest):
             sudo('twoliner')
         expected = """
 [%(prefix)s] sudo: oneliner
-[%(prefix)s] Login password: 
+[%(prefix)s] Login password for '%(user)s': 
 [%(prefix)s] out: sudo password:
 [%(prefix)s] out: Sorry, try again.
 [%(prefix)s] out: sudo password: 
@@ -373,7 +376,7 @@ class TestNetwork(FabricTest):
 [%(prefix)s] out: sudo password:
 [%(prefix)s] out: result1
 [%(prefix)s] out: result2
-""" % {'prefix': env.host_string}
+""" % {'prefix': env.host_string, 'user': env.user}
         eq_(expected[1:], sys.stdall.getvalue())
 
     @mock_streams('both')
@@ -400,12 +403,12 @@ class TestNetwork(FabricTest):
                 run('silent')
         expected = """
 [%(prefix)s] run: normal
-[%(prefix)s] Login password: 
+[%(prefix)s] Login password for '%(user)s': 
 [%(prefix)s] out: foo
 [%(prefix)s] run: silent
 [%(prefix)s] run: normal
 [%(prefix)s] out: foo
-""" % {'prefix': env.host_string}
+""" % {'prefix': env.host_string, 'user': env.user}
         eq_(expected[1:], sys.stdall.getvalue())
 
     @mock_streams('both')
@@ -428,12 +431,12 @@ class TestNetwork(FabricTest):
             run('twoliner')
         expected = """
 [%(prefix)s] run: oneliner
-[%(prefix)s] Login password: 
+[%(prefix)s] Login password for '%(user)s': 
 [%(prefix)s] out: result
 [%(prefix)s] run: twoliner
 [%(prefix)s] out: result1
 [%(prefix)s] out: result2
-""" % {'prefix': env.host_string}
+""" % {'prefix': env.host_string, 'user': env.user}
         eq_(expected[1:], sys.stdall.getvalue())
 
     @mock_streams('both')
@@ -457,14 +460,30 @@ class TestNetwork(FabricTest):
                 run('twoliner')
         expected = """
 [%(prefix)s] run: oneliner
-[%(prefix)s] Login password: 
+[%(prefix)s] Login password for '%(user)s': 
 result
 [%(prefix)s] run: twoliner
 result1
 result2
-""" % {'prefix': env.host_string}
+""" % {'prefix': env.host_string, 'user': env.user}
         eq_(expected[1:], sys.stdall.getvalue())
 
+    @server()
+    def test_env_host_set_when_host_prompt_used(self):
+        """
+        Ensure env.host is set during host prompting
+        """
+        copied_host_string = str(env.host_string)
+        fake = Fake('raw_input', callable=True).returns(copied_host_string)
+        env.host_string = None
+        env.host = None
+        with settings(hide('everything'), patched_input(fake)):
+            run("ls /")
+        # Ensure it did set host_string back to old value
+        eq_(env.host_string, copied_host_string)
+        # Ensure env.host is correct
+        eq_(env.host, normalize(copied_host_string)[1])
+
 
 def subtask():
     run("This should never execute")
@@ -566,6 +585,7 @@ class TestSSHConfig(FabricTest):
         Test-server connection using ssh_config values
         """
         with settings(
+            hide('everything'),
             ssh_config_path=support("testserver_ssh_config"),
             host_string='testserver',
         ):
