@@ -3,11 +3,14 @@ from __future__ import with_statement
 import os
 import shutil
 import sys
-import tempfile
 import types
 from contextlib import nested
 from StringIO import StringIO
 
+import unittest
+import random
+import types
+
 from nose.tools import raises, eq_
 from fudge import with_patched_object
 
@@ -17,6 +20,7 @@ from fabric.operations import require, prompt, _sudo_prefix, _shell_wrap, \
 from fabric.api import get, put, hide, show, cd, lcd, local, run, sudo
 from fabric.sftp import SFTP
 
+from fabric.decorators import with_settings
 from utils import *
 from server import (server, PORT, RESPONSES, FILES, PASSWORDS, CLIENT_PRIVKEY,
     USER, CLIENT_PRIVKEY_PASSPHRASE)
@@ -25,6 +29,7 @@ from server import (server, PORT, RESPONSES, FILES, PASSWORDS, CLIENT_PRIVKEY,
 # require()
 #
 
+
 def test_require_single_existing_key():
     """
     When given a single existing key, require() throws no exceptions
@@ -40,29 +45,26 @@ def test_require_multiple_existing_keys():
     require('version', 'sudo_prompt')
 
 
-@mock_streams('stderr')
-@raises(SystemExit)
+@aborts
 def test_require_single_missing_key():
     """
-    When given a single non-existent key, require() raises SystemExit
+    When given a single non-existent key, require() aborts
     """
     require('blah')
 
 
-@mock_streams('stderr')
-@raises(SystemExit)
+@aborts
 def test_require_multiple_missing_keys():
     """
-    When given multiple non-existent keys, require() raises SystemExit
+    When given multiple non-existent keys, require() aborts
     """
     require('foo', 'bar')
 
 
-@mock_streams('stderr')
-@raises(SystemExit)
+@aborts
 def test_require_mixed_state_keys():
     """
-    When given mixed-state keys, require() raises SystemExit
+    When given mixed-state keys, require() aborts
     """
     require('foo', 'version')
 
@@ -80,6 +82,28 @@ def test_require_mixed_state_keys_prints_missing_only():
         assert 'foo' in err
 
 
+@aborts
+def test_require_iterable_provided_by_key():
+    """
+    When given a provided_by iterable value, require() aborts
+    """
+    # 'version' is one of the default values, so we know it'll be there
+    def fake_providing_function():
+        pass
+    require('foo', provided_by=[fake_providing_function])
+
+
+@aborts
+def test_require_noniterable_provided_by_key():
+    """
+    When given a provided_by noniterable value, require() aborts
+    """
+    # 'version' is one of the default values, so we know it'll be there
+    def fake_providing_function():
+        pass
+    require('foo', provided_by=fake_providing_function)
+
+
 #
 # prompt()
 #
@@ -87,8 +111,9 @@ def test_require_mixed_state_keys_prints_missing_only():
 def p(x):
     print x,
 
+
 @mock_streams('stdout')
-@with_patched_object(sys.modules['__builtin__'], 'raw_input', p)
+@with_patched_input(p)
 def test_prompt_appends_space():
     """
     prompt() appends a single space when no default is given
@@ -99,7 +124,7 @@ def test_prompt_appends_space():
 
 
 @mock_streams('stdout')
-@with_patched_object(sys.modules['__builtin__'], 'raw_input', p)
+@with_patched_input(p)
 def test_prompt_with_default():
     """
     prompt() appends given default value plus one space on either side
@@ -108,7 +133,7 @@ def test_prompt_with_default():
     d = "default!"
     prompt(s, default=d)
     eq_(sys.stdout.getvalue(), "%s [%s] " % (s, d))
-    
+
 
 #
 # run()/sudo()
@@ -120,7 +145,7 @@ def test_sudo_prefix_with_user():
     """
     eq_(
         _sudo_prefix(user="foo"),
-        "%s -u \"foo\" " % (env.sudo_prefix % env.sudo_prompt)
+        "%s -u \"foo\" " % (env.sudo_prefix % env)
     )
 
 
@@ -128,9 +153,10 @@ def test_sudo_prefix_without_user():
     """
     _sudo_prefix() returns standard prefix when user is empty
     """
-    eq_(_sudo_prefix(user=None), env.sudo_prefix % env.sudo_prompt)
+    eq_(_sudo_prefix(user=None), env.sudo_prefix % env)
 
 
+@with_settings(use_shell=True)
 def test_shell_wrap():
     prefix = "prefix"
     command = "command"
@@ -149,6 +175,7 @@ def test_shell_wrap():
         del eq_.description
 
 
+@with_settings(use_shell=True)
 def test_shell_wrap_escapes_command_if_shell_is_true():
     """
     _shell_wrap() escapes given command if shell=True
@@ -243,28 +270,9 @@ class TestCombineStderr(FabricTest):
 #
 
 class TestFileTransfers(FabricTest):
-    def setup(self):
-        super(TestFileTransfers, self).setup()
-        self.tmpdir = tempfile.mkdtemp()
-
-    def teardown(self):
-        super(TestFileTransfers, self).teardown()
-        shutil.rmtree(self.tmpdir)
-
-    def path(self, *path_parts):
-        return os.path.join(self.tmpdir, *path_parts)
-
-    def exists_remotely(self, path):
-        return SFTP(env.host_string).exists(path)
-
-    def exists_locally(self, path):
-        return os.path.exists(path)
-
-
     #
     # get()
     #
-
     @server(files={'/home/user/.bashrc': 'bash!'}, home='/home/user')
     def test_get_relative_remote_dir_uses_home(self):
         """
@@ -274,8 +282,6 @@ class TestFileTransfers(FabricTest):
             # Another if-it-doesn't-error-out-it-passed test; meh.
             eq_(get('.bashrc', self.path()), [self.path('.bashrc')])
 
-
-
     @server()
     def test_get_single_file(self):
         """
@@ -287,7 +293,6 @@ class TestFileTransfers(FabricTest):
             get(remote, local)
         eq_contents(local, FILES[remote])
 
-
     @server()
     def test_get_sibling_globs(self):
         """
@@ -299,7 +304,6 @@ class TestFileTransfers(FabricTest):
         for remote in remotes:
             eq_contents(self.path(remote), FILES[remote])
 
-
     @server()
     def test_get_single_file_in_folder(self):
         """
@@ -310,7 +314,6 @@ class TestFileTransfers(FabricTest):
             get('folder', self.tmpdir)
         eq_contents(self.path(remote), FILES[remote])
 
-
     @server()
     def test_get_tree(self):
         """
@@ -322,7 +325,6 @@ class TestFileTransfers(FabricTest):
         for path, contents in leaves:
             eq_contents(self.path(path[1:]), contents)
 
-
     @server()
     def test_get_tree_with_implicit_local_path(self):
         """
@@ -342,7 +344,6 @@ class TestFileTransfers(FabricTest):
             if os.path.exists(dirname):
                 shutil.rmtree(dirname)
 
-
     @server()
     def test_get_absolute_path_should_save_relative(self):
         """
@@ -355,7 +356,6 @@ class TestFileTransfers(FabricTest):
         assert self.exists_locally(os.path.join(lpath, 'subfolder'))
         assert not self.exists_locally(os.path.join(lpath, 'tree/subfolder'))
 
-
     @server()
     def test_path_formatstr_nonrecursively_is_just_filename(self):
         """
@@ -367,7 +367,6 @@ class TestFileTransfers(FabricTest):
             get('/tree/subfolder/file3.txt', ltarget)
         assert self.exists_locally(os.path.join(lpath, 'file3.txt'))
 
-
     @server()
     @mock_streams('stderr')
     def _invalid_file_obj_situations(self, remote_path):
@@ -387,7 +386,6 @@ class TestFileTransfers(FabricTest):
         """
         self._invalid_file_obj_situations('/tree')
 
-
     @server()
     def test_get_single_file_absolutely(self):
         """
@@ -398,7 +396,6 @@ class TestFileTransfers(FabricTest):
             get(target, self.tmpdir)
         eq_contents(self.path(os.path.basename(target)), FILES[target])
 
-
     @server()
     def test_get_file_with_nonexistent_target(self):
         """
@@ -410,7 +407,6 @@ class TestFileTransfers(FabricTest):
             get(target, local)
         eq_contents(local, FILES[target])
 
-
     @server()
     @mock_streams('stderr')
     def test_get_file_with_existing_file_target(self):
@@ -426,7 +422,6 @@ class TestFileTransfers(FabricTest):
         assert "%s already exists" % local in sys.stderr.getvalue()
         eq_contents(local, FILES[target])
 
-
     @server()
     def test_get_file_to_directory(self):
         """
@@ -440,7 +435,6 @@ class TestFileTransfers(FabricTest):
             get(target, self.tmpdir)
         eq_contents(self.path(target), FILES[target])
 
-
     @server(port=2200)
     @server(port=2201)
     def test_get_from_multiple_servers(self):
@@ -465,7 +459,6 @@ class TestFileTransfers(FabricTest):
                         tmp, "127.0.0.1-%s" % port, 'file3.txt'
                     ))
 
-
     @server()
     def test_get_from_empty_directory_uses_cwd(self):
         """
@@ -478,7 +471,6 @@ class TestFileTransfers(FabricTest):
         for x in "file.txt file2.txt tree/file1.txt".split():
             assert os.path.exists(os.path.join(self.tmpdir, x))
 
-
     @server()
     def _get_to_cwd(self, arg):
         path = 'file.txt'
@@ -507,7 +499,6 @@ class TestFileTransfers(FabricTest):
         """
         self._get_to_cwd(None)
 
-
     @server()
     def test_get_should_accept_file_like_objects(self):
         """
@@ -519,7 +510,6 @@ class TestFileTransfers(FabricTest):
             get(target, fake_file)
         eq_(fake_file.getvalue(), FILES[target])
 
-
     @server()
     def test_get_interpolation_without_host(self):
         """
@@ -536,7 +526,6 @@ class TestFileTransfers(FabricTest):
             get('/folder/file3.txt', local_path)
             assert self.exists_locally(tmp + "bar/file3.txt")
 
-
     @server()
     def test_get_returns_list_of_local_paths(self):
         """
@@ -548,7 +537,6 @@ class TestFileTransfers(FabricTest):
         files = ['file1.txt', 'file2.txt', 'subfolder/file3.txt']
         eq_(map(lambda x: os.path.join(d, 'tree', x), files), retval)
 
-
     @server()
     def test_get_returns_none_for_stringio(self):
         """
@@ -557,18 +545,17 @@ class TestFileTransfers(FabricTest):
         with hide('everything'):
             eq_([], get('/file.txt', StringIO()))
 
-
     @server()
     def test_get_return_value_failed_attribute(self):
         """
-        get()'s return value should indicate any paths which failed to download.
+        get()'s return value should indicate any paths which failed to
+        download.
         """
         with settings(hide('everything'), warn_only=True):
             retval = get('/doesnt/exist', self.path())
         eq_(['/doesnt/exist'], retval.failed)
         assert not retval.succeeded
 
-
     @server()
     def test_get_should_not_use_windows_slashes_in_remote_paths(self):
         """
@@ -579,28 +566,22 @@ class TestFileTransfers(FabricTest):
             sftp = SFTP(env.host_string)
             eq_(sftp.glob(path), [path])
 
-
-
     #
     # put()
     #
-
     @server()
     def test_put_file_to_existing_directory(self):
         """
         put() a single file into an existing remote directory
         """
         text = "foo!"
-        local = self.path('foo.txt')
+        local = self.mkfile('foo.txt', text)
         local2 = self.path('foo2.txt')
-        with open(local, 'w') as fd:
-            fd.write(text)
         with hide('everything'):
             put(local, '/')
             get('/foo.txt', local2)
         eq_contents(local2, text)
 
-
     @server()
     def test_put_to_empty_directory_uses_cwd(self):
         """
@@ -619,7 +600,6 @@ class TestFileTransfers(FabricTest):
             get('foo.txt', local2)
         eq_contents(local2, text)
 
-
     @server()
     def test_put_from_empty_directory_uses_cwd(self):
         """
@@ -645,7 +625,6 @@ class TestFileTransfers(FabricTest):
         # Restore cwd
         os.chdir(old_cwd)
 
-
     @server()
     def test_put_should_accept_file_like_objects(self):
         """
@@ -663,7 +642,6 @@ class TestFileTransfers(FabricTest):
         # Sanity test of file pointer
         eq_(pointer, fake_file.tell())
 
-
     @server()
     @raises(ValueError)
     def test_put_should_raise_exception_for_nonexistent_local_path(self):
@@ -672,7 +650,6 @@ class TestFileTransfers(FabricTest):
         """
         put('thisfiledoesnotexist', '/tmp')
 
-
     @server()
     def test_put_returns_list_of_remote_paths(self):
         """
@@ -686,7 +663,6 @@ class TestFileTransfers(FabricTest):
             retval = put(f, p)
         eq_(retval, [p])
 
-
     @server()
     def test_put_returns_list_of_remote_paths_with_stringio(self):
         """
@@ -696,7 +672,6 @@ class TestFileTransfers(FabricTest):
         with hide('everything'):
             eq_(put(StringIO('contents'), f), [f])
 
-
     @server()
     def test_put_return_value_failed_attribute(self):
         """
@@ -708,12 +683,9 @@ class TestFileTransfers(FabricTest):
         eq_(["<StringIO>"], retval.failed)
         assert not retval.succeeded
 
-
-
     #
     # Interactions with cd()
     #
-
     @server()
     def test_cd_should_apply_to_put(self):
         """
@@ -728,7 +700,6 @@ class TestFileTransfers(FabricTest):
             put(local, f)
         assert self.exists_remotely('%s/%s' % (d, f))
 
-
     @server(files={'/tmp/test.txt': 'test'})
     def test_cd_should_apply_to_get(self):
         """
@@ -739,7 +710,6 @@ class TestFileTransfers(FabricTest):
             get('test.txt', local)
         assert os.path.exists(local)
 
-
     @server()
     def test_cd_should_not_apply_to_absolute_put(self):
         """
@@ -753,7 +723,6 @@ class TestFileTransfers(FabricTest):
         assert not self.exists_remotely('/tmp/test.txt')
         assert self.exists_remotely('/test.txt')
 
-
     @server(files={'/test.txt': 'test'})
     def test_cd_should_not_apply_to_absolute_get(self):
         """
@@ -764,7 +733,6 @@ class TestFileTransfers(FabricTest):
             get('/test.txt', local)
         assert os.path.exists(local)
 
-
     @server()
     def test_lcd_should_apply_to_put(self):
         """
@@ -780,7 +748,6 @@ class TestFileTransfers(FabricTest):
             put(f, '/')
         assert self.exists_remotely('/%s' % f)
 
-
     @server()
     def test_lcd_should_apply_to_get(self):
         """
