@@ -14,7 +14,7 @@ import shutil
 import sys
 import tempfile
 
-from fudge import Fake, patched_context, clear_expectations
+from fudge import Fake, patched_context, clear_expectations, with_patched_object
 from nose.tools import raises
 from nose import SkipTest
 
@@ -45,10 +45,13 @@ class FabricTest(object):
         # Temporary local file dir
         self.tmpdir = tempfile.mkdtemp()
 
+    def set_network(self):
+        env.update(to_dict('%s@%s:%s' % (USER, HOST, PORT)))
+
     def env_setup(self):
         # Set up default networking for test server
         env.disable_known_hosts = True
-        env.update(to_dict('%s@%s:%s' % (USER, HOST, PORT)))
+        self.set_network()
         env.password = PASSWORDS[USER]
         # Command response mocking is easier without having to account for
         # shell wrapping everywhere.
@@ -284,3 +287,9 @@ def path_prefix(module):
 
 def aborts(func):
     return raises(SystemExit)(mock_streams('stderr')(func))
+
+
+def _patched_input(func, fake):
+    return func(sys.modules['__builtin__'], 'raw_input', fake)
+patched_input = partial(_patched_input, patched_context)
+with_patched_input = partial(_patched_input, with_patched_object)
